/*******************************************************************************
				Neoinput
				========

		Copyright (C) 2010 Paul Kocialkowski

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <stdbool.h>

#include <fcntl.h>
#include <poll.h>

#include <linux/input.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include "server.h"
#include "server_config.h"

#include "input_read.h"
#include "input.h"
#include "config.h"

int input_count=0;
int input_touch=-1;
int input_accel=-1;
int input_button_aux=-1;
int input_button_power=-1;

bool input_read_open(int input)
{
	switch(input)
	{
		case DEVICE_ID_TOUCH:
			printf("Opening device: %s\n", server_config_value_get("device_touch"));
			input_touch=open(server_config_value_get("device_touch"), O_RDONLY);

			if(input_touch < 0)
			{
				printf("Error while opening input device\n");
				return 0;
			}
		break;
		case DEVICE_ID_ACCEL:
			printf("Opening device: %s\n", server_config_value_get("device_accel"));
			input_accel=open(server_config_value_get("device_accel"), O_RDONLY);

			if(input_accel < 0)
			{
				printf("Error while opening input device\n");
				return 0;
			}
		break;
		case DEVICE_ID_BUTTON_AUX:
			printf("Opening device: %s\n", server_config_value_get("device_button_aux"));
			input_button_aux=open(server_config_value_get("device_button_aux"), O_RDONLY);
			
			if(input_button_aux < 0)
			{
				printf("Error while opening input device\n");
				return 0;
			}
		break;
		case DEVICE_ID_BUTTON_POWER:
			printf("Opening device: %s\n", server_config_value_get("device_button_power"));
			input_button_power=open(server_config_value_get("device_button_power"), O_RDONLY);

			if(input_button_power < 0)
			{
				printf("Error while opening input device\n");
				return 0;
			}
		break;
	}

	return 1;
}

void input_read_event(void)
{
	struct input_event event;
	struct input_event_light event_light;
	struct pollfd poll_fd[input_count+1];
	int r, i, e=0;
	
	bool poll_quit=false;

	printf("Reading data...\n");

	poll_fd[0].fd=sock_client_fd;
	poll_fd[0].events = POLLIN;

	e=1;

	if(input_touch >= 0)
	{
		poll_fd[e].fd=input_touch;
		poll_fd[e].events = POLLIN;
		e++;
	}

	if(input_accel >= 0)
	{
		poll_fd[e].fd=input_accel;
		poll_fd[e].events = POLLIN;
		e++;
	}
	if(input_button_aux >= 0)
	{
		poll_fd[e].fd=input_button_aux;
		poll_fd[e].events = POLLIN;
		e++;
	}
	if(input_button_power >= 0)
	{
		poll_fd[e].fd=input_button_power;
		poll_fd[e].events = POLLIN;
		e++;
	}

	if(e != input_count+1)
	{
		printf("Error in count.\n");
		return;
	}

	while(!poll_quit)
	{
		/* Poll the inputs */
		r=poll(poll_fd, e, POLL_TIMEOUT);

		if(r == -1)
		{
			perror("Error in poll()");
			return;
		}	
		else if(r > 0)
		{
			if((poll_fd[0].revents & POLLIN) || (poll_fd[0].revents & POLLHUP) || (poll_fd[0].revents & POLLERR))
			{
				poll_quit=true;
				return;
			}

			for(i=1 ; i < e ; i++)
			{
				if(poll_fd[i].revents & POLLIN)
				{
					read(poll_fd[i].fd, &event, sizeof(struct input_event));

					event_light.type=event.type;
					event_light.code=event.code;
					event_light.value=event.value;

					if(!write(poll_fd[0].fd, &event_light, sizeof(event_light)))
					{
						poll_quit=true;
						return;
					}
				}
			}
		}
	}
}

void input_read_close(void)
{
	if(input_touch >= 0)
	{
		close(input_touch);
		input_touch=-1;
	}
	if(input_accel >= 0)
	{
		close(input_accel);
		input_accel=-1;
	}
	if(input_button_aux >= 0)
	{
		close(input_button_aux);
		input_button_aux=-1;
	}
	if(input_button_power >= 0)
	{
		close(input_button_power);
		input_button_power=-1;
	}
}
