/*******************************************************************************
				Neoinput
				========

		Copyright (C) 2011 Paul Kocialkowski

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"

#include "server.h"
#include "server_config.h"
#include "input_read.h"

void print_version(void)
{
	printf("%s_server %s, %s\n", SOFTWARE, VERSION, DESCRIPTION);
	printf("%s\n", COPYRIGHT);
}

void print_help(void)
{
	print_version();
	printf("\nUsage: %s_server [OPTION]...\n", SOFTWARE);
	printf("\nOptions:\n");
	printf("  --help        Shows this help message.\n");
	printf("  --version     Shows the program version.\n");
	printf("  -a [ADDRESS]  Specify the IP address to listen on. Default is INADDR_ANY.\n");
	printf("  -p [PORT]     Specify the port to listen on.\n");
	printf("  -c [FILE]     Specify the configuration file.\n");
}


int main(int argc, char **argv)
{
	int i, j;

	int server_port=0;
	char *server_ip=NULL;
	char *config_file=NULL;

	char append=0;

	/* Check args */

	for(i=1 ; i < argc ; i++)
	{
		if(argv[i][0] == '-')
		{
			if(argv[i][1] == '-')
			{
				if(strcmp(argv[i], "--help") == 0)
				{
					print_help();
					exit(0);
				}
				else if(strcmp(argv[i], "--version") == 0)
				{
					print_version();
					exit(0);
				}
				else
				{
					printf("Error: Unknown argument '%s'.\n", argv[i]);
					exit(1);
				}
			}
			else
			{
				for(j=1 ; j < strlen(argv[i]) ; j++)
				{
					if(append)
					{
						printf("Error: missing value for argument '%c'.\n", append);
						exit(1);
					}
					
					switch(argv[i][j])
					{
						case 'a':
							append='a';
						break;

						case 'p':
							append='p';
						break;

						case 'c':
							append='c';
						break;

						default:
							printf("Error: Unknown argument '%c'.\n", argv[i][j]);
						break;
					}
				}
			}
		}
		else
		{
			if(append)
			{
				if(append == 'p' && server_port == 0)
					server_port=atoi(argv[i]);
				else if(append == 'a' && server_ip == NULL)
					server_ip=argv[i];
				else if(append == 'c' && config_file == NULL)
					config_file=argv[i];
				else
				{
					printf("Error: Argument '%c' redifinition.\n", append);
					exit(1);
				}

				append=0;
			}
			else
			{
				printf("Error: Unknown argument '%s'.\n", argv[i]);
				exit(1);
			}
		}
	}

	if(config_file == NULL)
		config_file=SERVER_CONFIG;
	printf("Config file: %s\n", config_file);

	/* Read the configuration */
	server_config_read(config_file);

	if(server_ip == NULL)
		server_ip=server_config_value_get("ip");
	printf("Listen on IP: %s\n", server_ip);

	if(server_port == 0)
		server_port=atoi(server_config_value_get("port"));
	printf("Listen on port: %d\n", server_port);

	/* Open the server */
	server_open(server_ip, server_port);
	
	/* Accept connections */
	server_accept();

	/* Close the server */
	server_close();

	/* Close the configuration */
	server_config_close();
}
