/*******************************************************************************
				Neoinput
				========

		Copyright (C) 2010 Paul Kocialkowski

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <math.h>

#include <linux/input.h>
#include "input.h"

#include "config.h"
#include "input_read.h"
#include "server.h"

/* Sockets files descriptors */
int sock_fd, sock_client_fd;

/* Struct to store the server address */
struct sockaddr_in server_addr;

/* Struct to store the server address */
struct sockaddr_in client_addr;

void server_open(char *server_ip, int server_port)
{
	/* Start to open a socket */
	sock_fd = socket(AF_INET, SOCK_STREAM, 0);

	/* Check if the socket is ok */
	if(sock_fd == -1)
	{
		perror("Error in socket()");
		exit(1);
	}
	else
		printf("socket(): OK...\n");

	/* Server address family */
	server_addr.sin_family=AF_INET;

	/* Server port */
	server_addr.sin_port=htons(server_port);

	/* Server address */
	if(server_ip == NULL)
		server_addr.sin_addr.s_addr=INADDR_ANY;
	else
		server_addr.sin_addr.s_addr=inet_addr(server_ip);

	/* Set the Rest of the struct to 0 */

	memset(&(server_addr.sin_zero), 0, 8);

	/* Bind and check if bind is ok */ 
	if(bind(sock_fd, (struct sockaddr *)&server_addr, sizeof(struct sockaddr)) == -1)
	{
		perror("Error in bind()");
		exit(1);
	}
	else
		printf("bind(): OK...\n");
 
	/* Listen (10 connection max on queue) and cgeck if listen is ok */
	if(listen(sock_fd, 10) == -1)
	{
		perror("Error in listen()");
		exit(1);
	}
	else
		printf("listen(): OK...\n");
}

void server_accept(void)
{
	int sockaddr_in_size;
	int input_type;

	/* Get struct sockaddr_in size */
	sockaddr_in_size=sizeof(struct sockaddr_in);

	while(sock_client_fd=accept(sock_fd, (struct sockaddr *)&client_addr, &sockaddr_in_size))
	{
		if(sock_client_fd == -1)
		{
			perror("Error in accept()");
		}
		else
			printf("accept(): OK...\n");

		printf("New Connection\n");

		/* Get and open the devices we'll use */
		if(server_get_device())
		{
			/* Read and send input events */
			input_read_event();
		}

		/* Close the input device */
		input_read_close();

		printf("Connection closed\n");
		close(sock_client_fd);
	}
}

bool server_get_device(void)
{
	int input_device_use=0, input_device_id=0;
	int i;

	printf("Getting input type...\n");

	if(!read(sock_client_fd, &input_count, sizeof(input_count)))
	{
		perror("Error in read()");
		return false;
	}

	if(input_count)
		server_send_status(STATUS_OK);
	else
	{
		server_send_status(STATUS_ERROR);
		printf("Error: no input device to open.\n");
		return false;
	}

	if(!read(sock_client_fd, &input_device_use, sizeof(input_device_use)))
	{
		perror("Error in read()");
		return false;
	}

	for(i=0 ; i < input_count ; i++)
	{
		input_device_id=(input_device_use - (input_device_use % (int)pow(10, i))) / (int)pow(10, i) - 
		((input_device_use - (input_device_use % (int)pow(10, i+1))) / (int)pow(10, i+1))*10;

		if(!input_read_open(input_device_id))
		{
			server_send_status(STATUS_ERROR);
			return false;
		}
	}

	server_send_status(STATUS_OK);

	return true;
}

bool server_send_status(int status)
{
	printf("Sending status...\n");

	if(!write(sock_client_fd, &status, sizeof(status)))
	{
		perror("Error in write()");
		return false;
	}

	return true;
}

void server_close(void)
{
	close(sock_fd);
}
